from __future__ import absolute_import

import typing
import numpy as np
import scipy.sparse

from . import _oineus

from ._oineus import *

import warnings

try:
    from . import diff
except:
    warnings.warn("oineus.diff import failed, probably, because eagerpy is not installed")


# __all__ = ["compute_diagrams_ls", "compute_diagrams_and_v_ls", "get_boundary_matrix", "to_scipy_matrix", "is_reduced", "get_freudenthal_filtration"]


def to_scipy_matrix(sparse_cols, shape=None):
    if shape is None:
        shape = (len(sparse_cols), len(sparse_cols))
    row_ind = [j for i in range(len(sparse_cols)) for j in sparse_cols[i]]
    col_ind = [i for i in range(len(sparse_cols)) for _ in sparse_cols[i]]
    assert (len(row_ind) == len(col_ind))
    data = [1 for _ in range(len(row_ind))]
    return scipy.sparse.csc_matrix((data, (row_ind, col_ind)), shape=shape)


def is_reduced(a):
    lowest_ones = []
    for col_idx in range(a.shape[1]):
        if np.any(a[:, col_idx] % 2 == 1):
            lowest_ones.append(np.max(np.where(a[:, col_idx] % 2 == 1)))
    return len(lowest_ones) == len(set(lowest_ones))


def get_real_type(fil):
    if "_double" in str(type(fil)):
        return "double"
    elif "_float" in str(type(fil)):
        return "float"
    else:
        raise RuntimeError(f"Unknown type: {type(fil)}")


def get_type_dim(data: np.ndarray, points=False):
    if data.dtype == np.float32:
        type_part = "float"
    elif data.dtype == np.float64:
        type_part = "double"
    else:
        raise RuntimeError(f"Type not supported: {data.dtype}")

    if points:
        if data.ndim == 2 and data.shape[1] in [1, 2, 3, 4]:
            dim_part = str(data.shape[1])
        else:
            raise RuntimeError(f"Dimension not supported: shape = {data.shape}")
    else:
        if data.ndim in [1, 2, 3]:
            dim_part = str(data.ndim)
        else:
            raise RuntimeError(f"Dimension not supported: shape = {data.shape}")

    return type_part, dim_part


def get_freudenthal_filtration(data, negate, wrap, max_dim, n_threads):
    type_part, dim_part = get_type_dim(data)
    func = getattr(_oineus, f"get_fr_filtration_{type_part}_{dim_part}")
    return func(data, negate, wrap, max_dim, n_threads)

def get_freudenthal_filtration_and_critical_vertices(data, negate, wrap, max_dim, n_threads):
    type_part, dim_part = get_type_dim(data)
    func = getattr(_oineus, f"get_fr_filtration_and_critical_vertices_{type_part}_{dim_part}")
    fil, cv = func(data, negate, wrap, max_dim, n_threads)
    return fil, np.array(cv)


def get_vr_filtration_from_pwdists(pwdists, max_dim, max_radius, n_threads):
    type_part, _ = get_type_dim(pwdists, False)
    func = getattr(_oineus, f"get_vr_filtration_from_pwdists_{type_part}")
    return func(pwdists, max_dim, max_radius, n_threads)

def get_vr_filtration_and_critical_edges_from_pwdists(pwdists, max_dim, max_radius, n_threads):
    type_part, _ = get_type_dim(pwdists, False)
    func = getattr(_oineus, f"get_vr_filtration_and_critical_edges_from_pwdists_{type_part}")
    fil, edges = func(pwdists, max_dim, max_radius, n_threads)
    edges = np.array([[e.x, e.y] for e in edges])
    return fil, edges


def get_vr_filtration(points, max_dim, max_radius, n_threads):
    type_part, dim_part = get_type_dim(points, True)
    func = getattr(_oineus, f"get_vr_filtration_{type_part}_{dim_part}")
    return func(points, max_dim, max_radius, n_threads)

def get_vr_filtration_and_critical_edges(points, max_dim, max_radius, n_threads):
    type_part, dim_part = get_type_dim(points, True)
    func = getattr(_oineus, f"get_vr_filtration_and_critical_edges_{type_part}_{dim_part}")
    fil, edges = func(points, max_dim, max_radius, n_threads)
    edges = np.array([[e.x, e.y] for e in edges])
    return fil, edges


def get_boundary_matrix(data, negate, wrap, max_dim, n_threads):
    type_part, dim_part = get_type_dim(data)
    func = getattr(_oineus, f"get_boundary_matrix_{type_part}_{dim_part}")
    bm = func(data, negate, wrap, max_dim, n_threads)
    return to_scipy_matrix(bm)


def compute_diagrams_ls(data, negate, wrap, max_dim, params, include_inf_points, dualize):
    type_part, dim_part = get_type_dim(data)
    func = getattr(_oineus, f"compute_diagrams_ls_{type_part}_{dim_part}")
    return func(data, negate, wrap, max_dim, params, include_inf_points, dualize)


def get_denoise_target(d, fil, rv, eps, strat):
    type_part = get_real_type(fil)
    func = getattr(_oineus, f"get_denoise_target_{type_part}")
    return func(d, fil, rv, eps, strat)


def get_well_group_target(d, fil, rv, t):
    type_part = get_real_type(fil)
    func = getattr(_oineus, f"get_well_group_target_{type_part}")
    return func(d, fil, rv, t)


def get_ls_target_values_diagram_loss(d, dtv, fil):
    type_part = get_real_type(fil)
    func = getattr(_oineus, f"get_target_values_diagram_loss_{type_part}")
    return func(dtv, False)


def get_vr_target_values_diagram_loss(d, dtv, fil):
    type_part = get_real_type(fil)
    death_only = d == 0
    func = getattr(_oineus, f"get_target_values_diagram_loss_{type_part}")
    return func(dtv, death_only)



def get_ls_target_values_x(d, dtv, fil, decmp, decmp_coh, conflict_strategy):
    type_part = get_real_type(fil)
    func = getattr(_oineus, f"get_ls_target_values_x_{type_part}")
    return func(d, dtv, fil, decmp, decmp_coh, conflict_strategy, False)


def get_vr_target_values_x(d, dtv, fil, decmp, decmp_coh, conflict_strategy):
    type_part = get_real_type(fil)
    death_only = d == 0
    func = getattr(_oineus, f"get_vr_target_values_x_{type_part}")
    return func(d, dtv, fil, decmp, decmp_coh, conflict_strategy, death_only)



def get_ls_wasserstein_matching_target_values(dgm, fil, rv, d: int, q: float, mip: bool, mdp: bool):
    type_part = get_real_type(fil)
    func = getattr(_oineus, f"get_ls_wasserstein_matching_target_values_{type_part}")

    if type(dgm) is np.ndarray:
        DgmPt = getattr(_oineus, f"DiagramPoint_{type_part}")
        dgm_1 = []
        assert len(dgm.shape) == 2 and dgm.shape[1] == 2
        for p in dgm:
            dgm_1.append(DgmPt(p[0], p[1]))
        dgm = dgm_1

    return func(dgm, fil, rv, d, q, mip, mdp)



def get_bruelle_target(fil, rv, p, q, i_0, d, minimize, min_birth, max_death):
    type_part = get_real_type(fil)
    func = getattr(_oineus, f"get_bruelle_target_{type_part}")
    return func(fil, rv, p, q, i_0, d, minimize, min_birth, max_death)


def get_barycenter_target(fil, rv, d):
    type_part = get_real_type(fil)
    func = getattr(_oineus, f"get_barycenter_target_{type_part}")
    return func(fil, rv, d)


def get_nth_persistence(fil, rv, d, n):
    type_part = get_real_type(fil)
    func = getattr(_oineus, f"get_nth_persistence_{type_part}")
    return func(fil, rv, d, n)


def get_permutation(target_values, fil):
    if len(target_values) == 0:
        return {}
    type_part = get_real_type(fil)
    func = getattr(_oineus, f"get_permutation_{type_part}")
    return func(target_values, fil)


def list_to_filtration(simplex_list): #take a list which contains data for simplices and convert it to a filtration
    string_type = str(type(simplex_list[0][2]))
    if "int" in string_type:
        func = getattr(_oineus, f"list_to_filtration_int")
        return func(simplex_list)
    elif "float" in string_type:
        func = getattr(_oineus, f"list_to_filtration_float")
        return func(simplex_list)
    elif "double" in string_type:
        func = getattr(_oineus, f"list_to_filtration_double")
        return func(simplex_list)

def compute_kernel_image_cokernel_reduction(K_, L_, IdMap, n_threads): #
    string_type = str(type(K_[0][2]))
    func = getattr(_oineus, f"compute_kernel_image_cokernel_reduction_float")
    return func(K_, L_, IdMap, n_threads)


def get_ls_filtration(simplices: typing.List[typing.List[int]], vertex_values: np.ndarray, negate: bool, n_threads: int):
    if vertex_values.dtype == np.float32:
        func = getattr(_oineus, f"get_ls_filtration_float")
    elif vertex_values.dtype == np.float64:
        func = getattr(_oineus, f"get_ls_filtration_double")
    return func(simplices, vertex_values, negate, n_threads)


def compute_ker_im_cok_reduction_cyl(fil_2, fil_3):
    fil_min = _oineus.min_filtration(fil_2, fil_3)

    id_domain = fil_3.size() + fil_min.size() + 1
    id_codomain = id_domain + 1

    # id_domain: id of vertex at the top of the cylinder,
    # i.e., we multiply fil_3 with id_domain
    # id_codomain: id of vertex at the bottom of the cylinder
    # i.e, we multiply fil_min with id_codomain

    v0 = _oineus.Simplex(id_domain, [id_domain])
    v1 = _oineus.Simplex(id_codomain, [id_codomain])

    fil_cyl = _oineus.mapping_cylinder(fil_3, fil_min, v0, v1)

    # to get a subcomplex, we multiply each fil_3 with id_domain
    fil_3_prod = _oineus.multiply_filtration(fil_3, v0)

    params = _oineus.ReductionParams()
    params.kernel = params.cokernel = True
    params.image = False

    kicr_reduction = _oineus.KerImCokReducedProd_double(fil_cyl, fil_3_prod, params)
    return kicr_reduction

def compute_relative_diagrams(fil, rel, include_inf_points=True):
    type_part = get_real_type(fil)
    func = getattr(_oineus, f"compute_relative_diagrams_{type_part}")
    return func(fil, rel, include_inf_points)

#def compute_cokernel_diagrams(K_, L_, IdMap, n_threads): #
#    string_type = str(type(K_[0][2]))
#    func = getattr(_oineus, f"compute_cokernel_diagrams_float")
#    return func(K_, L_, IdMap, n_threads)
#

#def lists_to_paired_filtrations(simplex_list_1, simplex_list_2)



#!/usr/bin/env python3

import numpy as np
import torch
from matplotlib import pyplot as plt
import oineus as oin

def sample_data():
    # sample points from the unit circle
    # return points as differentiable torch tensor
    np.random.seed(1)

    num_points = 120
    noise_std_dev = 0.1

    angles = np.random.uniform(low=0, high=2*np.pi, size=num_points)
    x = np.cos(angles)
    y = np.sin(angles)

    x += np.random.normal(loc=0, scale=noise_std_dev, size=num_points)
    y += np.random.normal(loc=0, scale=noise_std_dev, size=num_points)

    pts = np.vstack((x, y)).T
    pts = torch.Tensor(pts)
    pts.requires_grad_(True)

    return pts


def topological_loss(pts: torch.Tensor, dim: int=1, n: int=2):
    pts_as_numpy = pts.clone().detach().numpy().astype(np.float64)
    fil, longest_edges = oin.get_vr_filtration_and_critical_edges(pts_as_numpy, max_dim=2, max_radius=9.0, n_threads=1)
    top_opt = oin.TopologyOptimizer(fil)

    # dgm = top_opt.compute_diagram(include_inf_points=False)
    # print(dgm.in_dimension(dim))

    eps = top_opt.get_nth_persistence(dim, n)
    print(f"{eps= }")

    indices, values = top_opt.simplify(eps, oin.DenoiseStrategy.BirthBirth, dim)

    critical_sets = top_opt.singletons(indices, values)
    crit_indices, crit_method_values = top_opt.combine_loss(critical_sets, oin.ConflictStrategy.Max)

    crit_indices = np.array(crit_indices, dtype=np.int32)

    dgm_method_edges = longest_edges[indices, :]
    dgm_method_edges_x, dgm_method_edges_y = dgm_method_edges[:, 0], dgm_method_edges[:, 1]

    dgm_method_values = torch.Tensor(values)

    crit_method_edges = longest_edges[crit_indices, :]
    crit_method_edges_x, crit_method_edges_y = crit_method_edges[:, 0], crit_method_edges[:, 1]

    crit_method_values = torch.Tensor(crit_method_values)

    if len(crit_method_edges_x) > 0:
        dgm_loss = torch.sum(torch.abs(torch.sum((pts[dgm_method_edges_x, :] - pts[dgm_method_edges_y, :])**2, axis=1) - dgm_method_values ** 2))
        top_loss = torch.sum(torch.abs(torch.sum((pts[crit_method_edges_x, :] - pts[crit_method_edges_y, :])**2, axis=1) - crit_method_values ** 2))
    else:
        top_loss = torch.zeros(())
        dgm_loss = torch.zeros(())

        top_loss.requires_grad_(True)
        dgm_loss.requires_grad_(True)
    return top_loss, dgm_loss


if __name__ == "__main__":
    draw = False
    use_critical_sets = False

    pts = sample_data()

    lr = 0.2

    opt = torch.optim.SGD([pts], lr=lr)

    for step_idx in range(5):
        opt.zero_grad()

        loss, dgm_loss = topological_loss(pts, dim=1, n=1)

        print(f"{step_idx = }, {loss = }, {dgm_loss = }")

        if draw:
            np_pts = pts.clone().detach().numpy()
            plt.scatter(np_pts[:, 0], np_pts[:, 1], color="green")

        if loss.item() == 0:
            break

        if use_critical_sets:
            loss.backward()
        else:
            dgm_loss.backward()

        opt.step()

        if draw:
            np_pts = pts.clone().detach().numpy()
            plt.scatter(np_pts[:, 0], np_pts[:, 1], color="red")
            plt.show()






#!python3

from icecream import ic
import numpy as np
import oineus as oin
import torch

n_pts = 50

np.random.seed(1)

pts_1 = np.random.uniform(low=0.0, high=5.0, size=n_pts * 2).reshape(n_pts, 2).astype(np.float64)
pts_2 = pts_1 + np.random.uniform(size=n_pts *2, low=-0.01, high=0.01).reshape(pts_1.shape)

# input: pts
# output: VR filtration and torch Tensor with the lengths
# of critical edges in filtration order, that is:
# fil.simplices()[sorted_idx].value == lengths[sorted_idx]
def get_vr_filtration_and_vals(pts, max_dim, max_radius):
    fil, edges = oin.get_vr_filtration_and_critical_edges(pts, max_dim=max_dim, max_radius=max_radius, n_threads=1)
    fil.reset_ids_to_sorted_ids()
    pts = torch.Tensor(pts)
    edge_start, edge_end = edges[:, 0], edges[:, 1]
    lengths = torch.sum((pts[edge_start, :] - pts[edge_end, :])**2, axis=1) ** 0.5

    return fil, lengths

max_dim = 2
max_radius = 1000.0

fil_1, lengths_1 = get_vr_filtration_and_vals(pts_1, max_dim, max_radius)
fil_2, lengths_2 = get_vr_filtration_and_vals(pts_2, max_dim, max_radius)

# create fil_min

simplices_1 = fil_1.simplices()
simplices_2 = fil_2.simplices()

fil_min_simplices = []

for sigma in simplices_1:
    min_sigma = oin.Simplex_double(sigma.sorted_id, sigma.vertices,
                                   min(sigma.value, fil_2.simplex_value_by_vertices(sigma.vertices)))
    fil_min_simplices.append(min_sigma)

fil_min = oin.Filtration_double(fil_min_simplices, set_ids=False)

# now simplices became reshuffled
# we must permute lengths_1 and lengths_2 accordingly
# id of simplex in fil_min is it's sorted_id in the original filtration

min_sorted_id_to_sorted_id_1 = fil_min.get_inv_sorting_permutation()

lengths_1_in_min_order = lengths_1[torch.LongTensor(min_sorted_id_to_sorted_id_1)]

min_simplices = fil_min.simplices()


min_sorted_id_to_sorted_id_2 = [ fil_2.get_sorted_id_by_vertices(min_sigma.vertices) for min_sigma in min_simplices ]

lengths_2_in_min_order = lengths_2[torch.LongTensor(min_sorted_id_to_sorted_id_2)]

for i, sigma in enumerate(min_simplices):
    s1 = simplices_1[fil_1.get_sorted_id_by_vertices(sigma.vertices)]
    s2 = simplices_2[fil_2.get_sorted_id_by_vertices(sigma.vertices)]
    assert np.abs(s1.value - lengths_1_in_min_order[i]) < 0.0001
    assert np.abs(s2.value - lengths_2_in_min_order[i]) < 0.0001


min_critical_values = torch.min(torch.stack((lengths_1_in_min_order, lengths_2_in_min_order)), axis=0)[0]

# verify that we got the critical values correctly
for sigma in min_simplices:
    assert torch.abs(sigma.value - min_critical_values[sigma.sorted_id]) < 0.00001


min_and_coned_simplices = min_simplices[:]
cone_v = fil_min.n_vertices()

# append cone vertex
# TODO: how to ensure that cone_vertex appears first?
# for now we can ignore it
min_and_coned_simplices.append(oin.Simplex_double(cone_v, [cone_v], 0.0))

simplex_id = len(min_and_coned_simplices)

# take cones over simplices from fil_1
for sigma in simplices_1:
    coned_sigma = sigma.join(new_vertex=cone_v, value=sigma.value, new_id=simplex_id)
    simplex_id += 1
    min_and_coned_simplices.append(coned_sigma)

fil_min_and_coned = oin.Filtration_double(min_and_coned_simplices, set_ids=False)

# in filtration order
min_and_coned_simplices = fil_min_and_coned.simplices()

for sigma in min_and_coned_simplices:
    if cone_v in sigma.vertices:
        if sigma.dim() == 0:
            continue
        # subtract additional 1 for cone vertex
        orig_sorted_sigma_id = sigma.id - len(fil_1) - 1
        assert(np.abs(lengths_1[orig_sorted_sigma_id] - sigma.value) < 0.00001)
    else:
        orig_sigma_id = sigma.id
        orig_sorted_sigma_id = fil_min.get_sorted_id_by_id(orig_sigma_id)
        assert(np.abs(min_critical_values[orig_sorted_sigma_id] - sigma.value) < 0.00001)

print(fil_min_and_coned)



#!/usr/bin/env python3

import numpy as np
from icecream import ic

import oineus as oin


def evaluate_func(x, y, z):
    return np.sin(x + 2 * y + 3 * z ** 2)


# Create a 16x16x16 grid
x = np.linspace(-np.pi, np.pi, 16)
y = np.linspace(-np.pi, np.pi, 16)
z = np.linspace(-np.pi, np.pi, 16)

# Generate a 3D meshgrid
X, Y, Z = np.meshgrid(x, y, z)

# Evaluate the function on the grid
f = evaluate_func(X, Y, Z)

fil, max_value_vertices = oin.get_freudenthal_filtration_and_critical_vertices(f, negate=False, wrap=False, max_dim=2, n_threads=8)

top_opt = oin.TopologyOptimizer(fil)

dim = 1
n = 4

dgm = top_opt.compute_diagram(include_inf_points=False)
eps = top_opt.get_nth_persistence(dim, 4)
indices, values = top_opt.simplify(eps, oin.DenoiseStrategy.BirthBirth, dim)
critical_sets = top_opt.singletons(indices, values)
crit_indices, crit_values = top_opt.combine_loss(critical_sets, oin.ConflictStrategy.Max)

crit_indices = np.array(crit_indices, dtype=np.int32)
crit_vertices = max_value_vertices[crit_indices]


#!python3

import numpy as np
import oineus as oin

# random points in space
np.random.seed(1)
n_points = 20
dim = 3
points = np.random.uniform(size=(n_points, dim))

# triangulate domain via Freudenthal and create lower star filtration
# negate: set to True to get upper-star filtration
# wrap: set to True to work on torus (periodic boundary conditions)
fil = oin.get_vr_filtration(points=points, max_dim=3, max_radius=2, n_threads=1)

cells = fil.cells()

# Vertices in cells are ids, not sorted_ids
print(f"Filtration with {len(cells)} cells created,\nvertex 0: {cells[0]},\nlast simplex: {cells[-1]}")

# no cohomology
dualize = False
# create VRU decomposition object, does not perform reduction yet
dcmp = oin.Decomposition(fil, dualize)

# reduction parameters
# relevant members:
# rp.clearing_opt --- whether you want to use clearing, True by default
# rp.compute_v: True by default
# rp.n_threads: number of threads to use, default is 1
# rp. compute_u: False by default (cannot do it in multi-threaded mode, so switch off just to be on the safe side)
rp = oin.ReductionParams()

# perform reduction
dcmp.reduce(rp)

# now we can acess V, R and U
# indices are sorted_ids of simplices == indices in fil.cells()
V = dcmp.v_data
print(f"Example of a V column: {V[-1]}, this chain contains cells:")
for sigma_idx in V[-1]:
    print(cells[sigma_idx])

# get diagram, including points at infinity
include_inf_points=True
dgm = dcmp.diagram(fil, include_inf_points)
# get only points of zero persistence
zero_dgm = dcmp.zero_pers_diagram(fil)

# diagram in dimension d is numpy array dgm[d], shape = (number of diagram points, 2)

for dim in range(dim):
    print(f"Diagram in dimension {dim}:")
    print(dgm[dim])




#!/usr/bin/env python3

import numpy as np
from icecream import ic

import oineus as oin
import torch


def evaluate_func(x, y, z):
    return np.sin(x + 1.8 * y + 3.1 * z ** 2)

def evaluate_func_template(x, y, z):
    return np.sin(x + 2 * y + 3 * z ** 2)


# Create a 16x16x16 grid
x = np.linspace(-np.pi, np.pi, 16)
y = np.linspace(-np.pi, np.pi, 16)
z = np.linspace(-np.pi, np.pi, 16)

# Generate a 3D meshgrid
X, Y, Z = np.meshgrid(x, y, z)

# Evaluate the function on the grid
f = evaluate_func(X, Y, Z)

# linearize 3D array to 1-D torch tensor
torch_f = torch.Tensor(f.reshape(-1))
torch_f.requires_grad_(True)

f_template = evaluate_func_template(X, Y, Z)

negate = False
wrap =False


fil, max_value_vertices = oin.get_freudenthal_filtration_and_critical_vertices(f, negate=negate, wrap=wrap, max_dim=2, n_threads=8)

top_opt = oin.TopologyOptimizer(fil)

dim = 1

rp = oin.ReductionParams()
rp.compute_u = False
rp.compute_v = False
rp.clearing_opt = True

template_dgms = oin.compute_diagrams_ls(f_template, negate=negate, wrap=wrap, max_dim=dim, include_inf_points=False, params=rp)
template_dgm = template_dgms.in_dimension(dim, as_numpy=False)
ic(template_dgm)


wass_q = 1.0

# if we only want to know where to move the points
indices, values = top_opt.match(template_dgm, dim, wass_q, False)
# if we want to get Wasserstein distance as well
(indices, values), distance = top_opt.match(template_dgm, dim, wass_q, True)
critical_sets = top_opt.singletons(indices, values)
crit_indices, crit_values = top_opt.combine_loss(critical_sets, oin.ConflictStrategy.Max)

crit_values = torch.Tensor(crit_values)

crit_indices = np.array(crit_indices, dtype=np.int32)
crit_vertices = max_value_vertices[crit_indices]
crit_vertices = torch.LongTensor(crit_vertices)

ic(crit_indices.shape, crit_vertices.shape, torch_f.shape)

opt = torch.optim.SGD([torch_f], lr=0.2)

opt.zero_grad()
top_loss = torch.sum((torch_f[crit_vertices] - crit_values) ** 2)
top_loss.backward()
opt.step()



#!python3

from icecream import ic
import numpy as np
import oineus as oin

n_pts = 5

pts = np.random.uniform(size=n_pts * 2).reshape(n_pts, 2).astype(np.float64)

fil_1 = oin.get_vr_filtration(pts, 1, 2.0, 1)
fil_2 = oin.get_vr_filtration(pts, 1, 2.0, 1)


simplices = fil_1.simplices()

fil_min_simplices = []

for sigma in simplices:
    min_sigma = oin.Simplex_double(sigma.vertices, min(sigma.value, fil_2.simplex_value_by_vertices(sigma.vertices)))
    fil_min_simplices.append(min_sigma)

fil_min = oin.Filtration_double(fil_min_simplices)

cone_v = fil_min.n_vertices()

# must make copy here! otherwise iteration in for loop below will never end,
# since we keep adding to the same list over which we iterate
coned_simplices = simplices[:]
simplex_id = fil_min.size()

for sigma in simplices:
    coned_sigma = sigma.join(new_vertex=cone_v, value=sigma.value, new_id=simplex_id)
    simplex_id += 1
    coned_simplices.append(coned_sigma)


fil_coned = oin.Filtration_double(coned_simplices, sort_only_by_dimension=True, set_ids=False)

print(fil_coned)



#!python3

import numpy as np
import oineus as oin

# here we create a simple example (triangle) with vertices
# v0 (critical value 0.2)
# v1 (critical value 0.1)
# v3 (critical value 0.3)
# edges: e1 = [v0, v1]
#

v0 = oin.Simplex_double([0], 0.2)
v1 = oin.Simplex_double([1], 0.1)
v2 = oin.Simplex_double([2], 0.3)

# indices of vertices should be sorted
e1 = oin.Simplex_double([0, 1], 0.9)
e2 = oin.Simplex_double([0, 2], 0.5)
e3 = oin.Simplex_double([1, 2], 0.8)

t = oin.Simplex_double([0, 1, 2], 1.0)

simplices = [v0, v1, v2, e1, e2, e3, t]

negate = False
n_threads = 1

# constructor will sort simplices and assign sorted_ids
fil = oin.Filtration_double(simplices, negate, n_threads)

fil_simplices = fil.simplices()

for sigma in fil.simplices():
    print(sigma)

# no cohomology
dualize = False
# create VRU decomposition object, does not perform reduction yet
dcmp = oin.Decomposition(fil, dualize)

# reduction parameters
# relevant members:
# rp.clearing_opt --- whether you want to use clearing, True by default
# rp.compute_v: True by default
# rp.n_threads: number of threads to use, default is 1
# rp. compute_u: False by default (cannot do it in multi-threaded mode, so switch off just to be on the safe side)
rp = oin.ReductionParams()

# perform reduction
dcmp.reduce(rp)

# get diagram, including points at infinity
include_inf_points=True
dgm = dcmp.diagram(fil, include_inf_points)

# diagram in dimension d is numpy array dgm[d], shape = (number of diagram points, 2)

for dim in [0, 1]:
    print(f"Diagram in dimension {dim}:")
    print(dgm[dim])

